VERSION = '3'
CLIENT_VERSION = '0.0.1.a0'
PUBLIC_ENDPOINT = 'https://api.azionapi.net'
USER_AGENT = "azion-api-client/%s" %  CLIENT_VERSION

DEFAULT_PAGE_SIZE = 100
DEFAULT_PARAMS = {'page_size': DEFAULT_PAGE_SIZE}
